library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use ieee.math_real.all;

entity Forum_FIR_bench is
end Forum_FIR_bench;

architecture Behavioral of Forum_FIR_bench is

component Forum_FIR is Port(
	clk: in std_logic;
	data_in: in std_logic_vector(15 downto 0);
	data_in_valid: in std_logic;
	data_out: out std_logic_vector(15 downto 0);
	data_out_valid: out std_logic);
end component;

signal clk: std_logic:='0';
signal data_in: std_logic_vector(15 downto 0):=(others => '0');
signal data_in_valid: std_logic:='0';
signal data_out: std_logic_vector(15 downto 0):=(others => '0');
signal data_out_valid: std_logic:='0';

type rom128x16 is array (0 to 63) of signed(15 downto 0); 
signal sinus_rom : rom128x16;

signal counter: unsigned(5 downto 0):=(others => '0');
signal samplecounter: unsigned(5 downto 0):=(others => '0');
signal speed: unsigned(4 downto 0):="00001";

signal clk_divider: integer range 0 to 7:=0;

begin

tabel : for I in 0 to 63 generate
	sinus_rom(I) <= to_signed(integer(sin(2.0*MATH_PI*(real(I)+0.5)/64.0)*32767.5),16);
end generate;

uut: Forum_FIR port map(
	clk => clk,
	data_in => data_in,
	data_in_valid => data_in_valid,
	data_out => data_out,
	data_out_valid => data_out_valid);

clk <= not clk after 5 ns;

process begin
	wait until rising_edge(clk);
	clk_divider <= clk_divider +1;
	data_in_valid <= '0';
	if clk_divider = 7 then
		data_in_valid <= '1';
		clk_divider <= 0;
		samplecounter <= samplecounter +1;
		counter <= counter + speed;
		if samplecounter = 63 then
			speed <= speed+1;
		end if;
		data_in <= std_logic_vector(sinus_rom(to_integer(counter)));
		if speed = 31 and samplecounter = 63 then
			wait;
		end if;
	end if;
end process;

end Behavioral;
